#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'json'
require 'open3'
require 'puppet'

class StopAdminServerTask

  STOP_STATES  = [
    'SHUTTING-DOWN',
    'SUSPENING',
    'FORCE_SUSPEND',
    'SHUTDOWN'
  ]
  
  def self.run
    instance = self.new
    instance.run
  end

  def run()
    begin
      fetch_input()
      execute()
      sync()
    rescue
      raise
    ensure
      report()
    end
  end

  def fetch_input
    params  = JSON.parse(STDIN.read)
    if params.keys.include?('_installdir')
      #
      # Use loadpath specfied in the params
      #
      $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
      $LOAD_PATH << "#{params['_installdir']}/wls_config/lib"
    else
      begin
        Puppet.initialize_settings
      rescue Puppet::DevError
        # When run in Rspec, this is called multiple times
      end
    end
    require 'easy_type'
    require 'puppet_x/enterprisemodules/weblogic/core'
    @domain       = params.fetch('domain')       { 'default'}
    @admin_server = params.fetch('admin_server') { 'AdminServer'}
    @wait         = params.fetch('wait')         { true}.to_s == 'true'
    @timeout      = params.fetch('timeout')      { 300 }.to_i
    @between      = params.fetch('between_time') { 10}.to_i
    @result       = {}
  end

  def report()
    puts @result.to_json
  end

  def execute
    if admin_server_running?
      @wls_config = Puppet_X::EnterpriseModules::WebLogic::Tree.new(@domain)
      begin
        # Check if the domain exists
        @wls_config.validate
      rescue ArgumentError => e
        @result[:_error] = { 
          msg: e.message,
          kind: "enterprisemodules-wls_config/unknown_domain"
        }
        raise e
      end
      @wls_config.stop_server(@admin_server)
      @result['stopped_managed_servers'] = [@admin_server]
    else
      @result['already_stopped_managed_servers'] = [@admin_server]
    end
  end

  def sync
    if @wait
      begin
        Timeout::timeout(@timeout) do
          loop do
            break unless admin_server_running?
            sleep(@between)
          end
        end
      rescue => e
        @result[:_error] = { 
          msg: e.message,
          kind: "enterprisemodules-wls_config/timeout"
        }
        raise e
      end
    end
  end

  def admin_server_running?
    `pgrep -f "weblogic.Name=#{@admin_server}"` != ''
  end

end

StopAdminServerTask.run if __FILE__ == $0